using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;
using Csla.Validation;
using System.Data.Common;


namespace JMDCMS.BusinessObjects
{ 
	[Serializable()] 
	public abstract class JmdUserBase<T> : Csla.BusinessBase<T>
		where T : JmdUserBase<T>
	{
		#region Business Properties and Methods

		//declare members
		protected int _id = 0;
		protected int _siteId = 0;
		protected string _userName = string.Empty;
		protected string _userDisplayName = string.Empty;
		protected string _passwordHash = string.Empty;
		protected string _salt = string.Empty;
		protected int _isInternal = 0;
		protected string _email = string.Empty;
		protected int _status = 0;
		protected SmartDate _creationDate = new SmartDate(DateTime.Today);
		protected SmartDate _validFrom = new SmartDate(DateTime.Today);
		protected SmartDate _validTo = new SmartDate(DateTime.Today);
		protected string _activationKey = string.Empty;

		[System.ComponentModel.DataObjectField(true, true)]
		public virtual int Id
		{
			get
			{
				CanReadProperty("Id", true);
				return _id;
			}
		}

		public virtual int SiteId
		{
			get
			{
				CanReadProperty("SiteId", true);
				return _siteId;
			}
			set
			{
				CanWriteProperty("SiteId", true);
				if (!_siteId.Equals(value))
				{
					_siteId = value;
					PropertyHasChanged("SiteId");
				}
			}
		}

		public virtual string UserName
		{
			get
			{
				CanReadProperty("UserName", true);
				return _userName;
			}
			set
			{
				CanWriteProperty("UserName", true);
				if (value == null) value = string.Empty;
				if (!_userName.Equals(value))
				{
					_userName = value;
					PropertyHasChanged("UserName");
				}
			}
		}

		public virtual string UserDisplayName
		{
			get
			{
				CanReadProperty("UserDisplayName", true);
				return _userDisplayName;
			}
			set
			{
				CanWriteProperty("UserDisplayName", true);
				if (value == null) value = string.Empty;
				if (!_userDisplayName.Equals(value))
				{
					_userDisplayName = value;
					PropertyHasChanged("UserDisplayName");
				}
			}
		}

		public virtual string PasswordHash
		{
			get
			{
				CanReadProperty("PasswordHash", true);
				return _passwordHash;
			}
			set
			{
				CanWriteProperty("PasswordHash", true);
				if (value == null) value = string.Empty;
				if (!_passwordHash.Equals(value))
				{
					_passwordHash = value;
					PropertyHasChanged("PasswordHash");
				}
			}
		}

		public virtual string Salt
		{
			get
			{
				CanReadProperty("Salt", true);
				return _salt;
			}
			set
			{
				CanWriteProperty("Salt", true);
				if (value == null) value = string.Empty;
				if (!_salt.Equals(value))
				{
					_salt = value;
					PropertyHasChanged("Salt");
				}
			}
		}

		public virtual int IsInternal
		{
			get
			{
				CanReadProperty("IsInternal", true);
				return _isInternal;
			}
			set
			{
				CanWriteProperty("IsInternal", true);
				if (!_isInternal.Equals(value))
				{
					_isInternal = value;
					PropertyHasChanged("IsInternal");
				}
			}
		}

		public virtual string Email
		{
			get
			{
				CanReadProperty("Email", true);
				return _email;
			}
			set
			{
				CanWriteProperty("Email", true);
				if (value == null) value = string.Empty;
				if (!_email.Equals(value))
				{
					_email = value;
					PropertyHasChanged("Email");
				}
			}
		}

		public virtual int Status
		{
			get
			{
				CanReadProperty("Status", true);
				return _status;
			}
			set
			{
				CanWriteProperty("Status", true);
				if (!_status.Equals(value))
				{
					_status = value;
					PropertyHasChanged("Status");
				}
			}
		}

		public virtual DateTime CreationDate
		{
			get
			{
				CanReadProperty("CreationDate", true);
				return _creationDate.Date;
			}
		}

		public virtual string CreationDateString
		{
			get
			{
				CanReadProperty("CreationDateString", true);
				return _creationDate.Text;
			}
			set
			{
				CanWriteProperty("CreationDateString", true);
				if (value == null) value = string.Empty;
				if (!_creationDate.Equals(value))
				{
					_creationDate.Text = value;
					PropertyHasChanged("CreationDateString");
				}
			}
		}

		public virtual DateTime ValidFrom
		{
			get
			{
				CanReadProperty("ValidFrom", true);
				return _validFrom.Date;
			}
		}

		public virtual string ValidFromString
		{
			get
			{
				CanReadProperty("ValidFromString", true);
				return _validFrom.Text;
			}
			set
			{
				CanWriteProperty("ValidFromString", true);
				if (value == null) value = string.Empty;
				if (!_validFrom.Equals(value))
				{
					_validFrom.Text = value;
					PropertyHasChanged("ValidFromString");
				}
			}
		}

		public virtual DateTime ValidTo
		{
			get
			{
				CanReadProperty("ValidTo", true);
				return _validTo.Date;
			}
		}

		public virtual string ValidToString
		{
			get
			{
				CanReadProperty("ValidToString", true);
				return _validTo.Text;
			}
			set
			{
				CanWriteProperty("ValidToString", true);
				if (value == null) value = string.Empty;
				if (!_validTo.Equals(value))
				{
					_validTo.Text = value;
					PropertyHasChanged("ValidToString");
				}
			}
		}

		public virtual string ActivationKey
		{
			get
			{
				CanReadProperty("ActivationKey", true);
				return _activationKey;
			}
			set
			{
				CanWriteProperty("ActivationKey", true);
				if (value == null) value = string.Empty;
				if (!_activationKey.Equals(value))
				{
					_activationKey = value;
					PropertyHasChanged("ActivationKey");
				}
			}
		}
 
		protected override object GetIdValue()
		{
			return _id;
		}

		#endregion //Business Properties and Methods

		#region Validation Rules
		protected virtual void AddCustomRules()
		{
			//hook for adding custom/non-generated rules
		}

		protected virtual void AddCommonRules()
		{
			//
			// UserName
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "UserName");
			ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("UserName", 50));
			//
			// UserDisplayName
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "UserDisplayName");
			ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("UserDisplayName", 50));
			//
			// PasswordHash
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "PasswordHash");
			ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("PasswordHash", 50));
			//
			// Salt
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "Salt");
			ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("Salt", 50));
			//
			// Email
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "Email");
			ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("Email", 50));
			//
			// CreationDate
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "CreationDateString");
			//
			// ValidFrom
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "ValidFromString");
			//
			// ValidTo
			//
			ValidationRules.AddRule(CommonRules.StringRequired, "ValidToString");
			//
			// ActivationKey
			//
			ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("ActivationKey", 100));
		}

		protected override void AddBusinessRules()
		{
			AddCommonRules();
			AddCustomRules();
		}
		#endregion //Validation Rules

		#region Factory Methods
		#endregion //Factory Methods

		#region Data Access

		#region Criteria

		[Serializable()]
		protected class Criteria : Csla.CriteriaBase
		{
			public int Id;

			public Criteria(int id) : base(typeof(T))
			{
				this.Id = id;
			}
		}

		#endregion //Criteria

		#region Data Access - Create
		[RunLocal]
		protected override void DataPortal_Create()
		{
			ValidationRules.CheckRules();
		}

		#endregion //Data Access - Create

		#region Data Access - Fetch
		private void DataPortal_Fetch(Criteria criteria)
		{
            //Database db = DatabaseFactory.CreateDatabase();
            //ExecuteFetch(db, criteria);
		}

		protected virtual void ExecuteFetch(Criteria criteria)
		{

            //string sqlCommand = "GetJmdUser";

            //using (DbCommand dbCommand = db.GetStoredProcCommand(sqlCommand))
            //{
            //    db.AddInParameter(dbCommand, "ID", DbType.Guid, criteria.Id);
            //    using (SafeDataReader dr = new SafeDataReader(db.ExecuteReader(dbCommand)))
            //    {
            //        FetchObject(dr);
            //        ValidationRules.CheckRules();
            //        //load child object(s)
            //        FetchChildren(dr);
            //    }
            //}
		}


		protected virtual void FetchObject(SafeDataReader dr)
		{
			dr.Read();
			_id = dr.GetInt32("ID");
			_siteId = dr.GetInt32("SITE_ID");
			_userName = dr.GetString("USER_NAME");
			_userDisplayName = dr.GetString("USER_DISPLAY_NAME");
			_passwordHash = dr.GetString("PASSWORD_HASH");
			_salt = dr.GetString("SALT");
			_isInternal = dr.GetInt32("IS_INTERNAL");
			_email = dr.GetString("EMAIL");
			_status = dr.GetInt32("STATUS");
			_creationDate = dr.GetSmartDate("CREATION_DATE", _creationDate.EmptyIsMin);
			_validFrom = dr.GetSmartDate("VALID_FROM", _validFrom.EmptyIsMin);
			_validTo = dr.GetSmartDate("VALID_TO", _validTo.EmptyIsMin);
			_activationKey = dr.GetString("ACTIVATION_KEY");
		}

		protected virtual void FetchChildren(SafeDataReader dr)
		{
		}
		#endregion //Data Access - Fetch

		#region Data Access - Insert
		protected override void DataPortal_Insert()
		{
            //using (SqlConnection cn = new SqlConnection(Database.JMDCMSConnection))
            //{
            //    cn.Open();

            //    ExecuteInsert(cn);

            //    //update child object(s)
            //    UpdateChildren(cn);
            //}//using

		}

		protected virtual void ExecuteInsert(SqlConnection cn)
		{
			using (SqlCommand cm = cn.CreateCommand())
			{
				cm.CommandType = CommandType.StoredProcedure;
				cm.CommandText = "AddJmdUser";

				AddInsertParameters(cm);

				cm.ExecuteNonQuery();

				_id = (int)cm.Parameters["@NewID"].Value;
			}//using
		}

		protected virtual void AddInsertParameters(SqlCommand cm)
		{
			cm.Parameters.AddWithValue("@SITE_ID", _siteId);
			cm.Parameters.AddWithValue("@USER_NAME", _userName);
			cm.Parameters.AddWithValue("@USER_DISPLAY_NAME", _userDisplayName);
			cm.Parameters.AddWithValue("@PASSWORD_HASH", _passwordHash);
			cm.Parameters.AddWithValue("@SALT", _salt);
			cm.Parameters.AddWithValue("@IS_INTERNAL", _isInternal);
			cm.Parameters.AddWithValue("@EMAIL", _email);
			cm.Parameters.AddWithValue("@STATUS", _status);
			cm.Parameters.AddWithValue("@CREATION_DATE", _creationDate.DBValue);
			cm.Parameters.AddWithValue("@VALID_FROM", _validFrom.DBValue);
			cm.Parameters.AddWithValue("@VALID_TO", _validTo.DBValue);
			if (_activationKey.Length > 0)
				cm.Parameters.AddWithValue("@ACTIVATION_KEY", _activationKey);
			else
				cm.Parameters.AddWithValue("@ACTIVATION_KEY", DBNull.Value);
			cm.Parameters.AddWithValue("@NewID", _id);
			cm.Parameters["@NewID"].Direction = ParameterDirection.Output;
		}
		#endregion //Data Access - Insert

		#region Data Access - Update
		protected override void DataPortal_Update()
		{
            //using (SqlConnection cn = new SqlConnection(Database.JMDCMSConnection))
            //{
            //    cn.Open();

            //    if (base.IsDirty)
            //    {
            //        ExecuteUpdate(cn);
            //    }

            //    //update child object(s)
            //    UpdateChildren(cn);
            //}//using

		}

		protected virtual void ExecuteUpdate(SqlConnection cn)
		{
			using (SqlCommand cm = cn.CreateCommand())
			{
				cm.CommandType = CommandType.StoredProcedure;
				cm.CommandText = "UpdateJmdUser";

				AddUpdateParameters(cm);

				cm.ExecuteNonQuery();

			}//using
		}

		protected virtual void AddUpdateParameters(SqlCommand cm)
		{
			cm.Parameters.AddWithValue("@ID", _id);
			cm.Parameters.AddWithValue("@SITE_ID", _siteId);
			cm.Parameters.AddWithValue("@USER_NAME", _userName);
			cm.Parameters.AddWithValue("@USER_DISPLAY_NAME", _userDisplayName);
			cm.Parameters.AddWithValue("@PASSWORD_HASH", _passwordHash);
			cm.Parameters.AddWithValue("@SALT", _salt);
			cm.Parameters.AddWithValue("@IS_INTERNAL", _isInternal);
			cm.Parameters.AddWithValue("@EMAIL", _email);
			cm.Parameters.AddWithValue("@STATUS", _status);
			cm.Parameters.AddWithValue("@CREATION_DATE", _creationDate.DBValue);
			cm.Parameters.AddWithValue("@VALID_FROM", _validFrom.DBValue);
			cm.Parameters.AddWithValue("@VALID_TO", _validTo.DBValue);
			if (_activationKey.Length > 0)
				cm.Parameters.AddWithValue("@ACTIVATION_KEY", _activationKey);
			else
				cm.Parameters.AddWithValue("@ACTIVATION_KEY", DBNull.Value);
		}

		protected virtual void UpdateChildren(SqlConnection cn)
		{
		}
		#endregion //Data Access - Update

		#region Data Access - Delete
		protected override void DataPortal_DeleteSelf()
		{
			DataPortal_Delete(new Criteria(_id));
		}

		private void DataPortal_Delete(Criteria criteria)
		{
            //using (SqlConnection cn = new SqlConnection(Database.JMDCMSConnection))
            //{
            //    cn.Open();

            //    ExecuteDelete(cn, criteria);

            //}//using

		}

		protected virtual void ExecuteDelete(SqlConnection cn, Criteria criteria)
		{
			using (SqlCommand cm = cn.CreateCommand())
			{
				cm.CommandType = CommandType.StoredProcedure;
				cm.CommandText = "DeleteJmdUser";

				cm.Parameters.AddWithValue("@ID", criteria.Id);

				cm.ExecuteNonQuery();
			}//using
		}
		#endregion //Data Access - Delete
		#endregion //Data Access
	}
}


