using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;
using Csla.Validation;
using System.Data.Common;


namespace JMDCMS.BusinessObjects
{
    [Serializable()]
    public class _JmdUser : Csla.BusinessBase<_JmdUser>
    {
        #region Business Properties and Methods

        //declare members
        private int _id = 0;
        private int _siteId = 0;
        private string _userName = string.Empty;
        private string _userDisplayName = string.Empty;
        private string _passwordHash = string.Empty;
        private string _salt = string.Empty;
        private int _isInternal = 0;
        private string _email = string.Empty;
        private int _status = 0;
        private SmartDate _creationDate = new SmartDate(DateTime.Today);
        private SmartDate _validFrom = new SmartDate(DateTime.Today);
        private SmartDate _validTo = new SmartDate(DateTime.Today);
        private string _activationKey = string.Empty;

        [System.ComponentModel.DataObjectField(true, true)]
        public int Id
        {
            get
            {
                CanReadProperty("Id", true);
                return _id;
            }
        }

        public int SiteId
        {
            get
            {
                CanReadProperty("SiteId", true);
                return _siteId;
            }
            set
            {
                CanWriteProperty("SiteId", true);
                if (!_siteId.Equals(value))
                {
                    _siteId = value;
                    PropertyHasChanged("SiteId");
                }
            }
        }

        public string UserName
        {
            get
            {
                CanReadProperty("UserName", true);
                return _userName;
            }
            set
            {
                CanWriteProperty("UserName", true);
                if (value == null) value = string.Empty;
                if (!_userName.Equals(value))
                {
                    _userName = value;
                    PropertyHasChanged("UserName");
                }
            }
        }

        public string UserDisplayName
        {
            get
            {
                CanReadProperty("UserDisplayName", true);
                return _userDisplayName;
            }
            set
            {
                CanWriteProperty("UserDisplayName", true);
                if (value == null) value = string.Empty;
                if (!_userDisplayName.Equals(value))
                {
                    _userDisplayName = value;
                    PropertyHasChanged("UserDisplayName");
                }
            }
        }

        public string PasswordHash
        {
            get
            {
                CanReadProperty("PasswordHash", true);
                return _passwordHash;
            }
            set
            {
                CanWriteProperty("PasswordHash", true);
                if (value == null) value = string.Empty;
                if (!_passwordHash.Equals(value))
                {
                    _passwordHash = value;
                    PropertyHasChanged("PasswordHash");
                }
            }
        }

        public string Salt
        {
            get
            {
                CanReadProperty("Salt", true);
                return _salt;
            }
            set
            {
                CanWriteProperty("Salt", true);
                if (value == null) value = string.Empty;
                if (!_salt.Equals(value))
                {
                    _salt = value;
                    PropertyHasChanged("Salt");
                }
            }
        }

        public int IsInternal
        {
            get
            {
                CanReadProperty("IsInternal", true);
                return _isInternal;
            }
            set
            {
                CanWriteProperty("IsInternal", true);
                if (!_isInternal.Equals(value))
                {
                    _isInternal = value;
                    PropertyHasChanged("IsInternal");
                }
            }
        }

        public string Email
        {
            get
            {
                CanReadProperty("Email", true);
                return _email;
            }
            set
            {
                CanWriteProperty("Email", true);
                if (value == null) value = string.Empty;
                if (!_email.Equals(value))
                {
                    _email = value;
                    PropertyHasChanged("Email");
                }
            }
        }

        public int Status
        {
            get
            {
                CanReadProperty("Status", true);
                return _status;
            }
            set
            {
                CanWriteProperty("Status", true);
                if (!_status.Equals(value))
                {
                    _status = value;
                    PropertyHasChanged("Status");
                }
            }
        }

        public DateTime CreationDate
        {
            get
            {
                CanReadProperty("CreationDate", true);
                return _creationDate.Date;
            }
        }

        public string CreationDateString
        {
            get
            {
                CanReadProperty("CreationDateString", true);
                return _creationDate.Text;
            }
            set
            {
                CanWriteProperty("CreationDateString", true);
                if (value == null) value = string.Empty;
                if (!_creationDate.Equals(value))
                {
                    _creationDate.Text = value;
                    PropertyHasChanged("CreationDateString");
                }
            }
        }

        public DateTime ValidFrom
        {
            get
            {
                CanReadProperty("ValidFrom", true);
                return _validFrom.Date;
            }
        }

        public string ValidFromString
        {
            get
            {
                CanReadProperty("ValidFromString", true);
                return _validFrom.Text;
            }
            set
            {
                CanWriteProperty("ValidFromString", true);
                if (value == null) value = string.Empty;
                if (!_validFrom.Equals(value))
                {
                    _validFrom.Text = value;
                    PropertyHasChanged("ValidFromString");
                }
            }
        }

        public DateTime ValidTo
        {
            get
            {
                CanReadProperty("ValidTo", true);
                return _validTo.Date;
            }
        }

        public string ValidToString
        {
            get
            {
                CanReadProperty("ValidToString", true);
                return _validTo.Text;
            }
            set
            {
                CanWriteProperty("ValidToString", true);
                if (value == null) value = string.Empty;
                if (!_validTo.Equals(value))
                {
                    _validTo.Text = value;
                    PropertyHasChanged("ValidToString");
                }
            }
        }

        public string ActivationKey
        {
            get
            {
                CanReadProperty("ActivationKey", true);
                return _activationKey;
            }
            set
            {
                CanWriteProperty("ActivationKey", true);
                if (value == null) value = string.Empty;
                if (!_activationKey.Equals(value))
                {
                    _activationKey = value;
                    PropertyHasChanged("ActivationKey");
                }
            }
        }

        protected override object GetIdValue()
        {
            return _id;
        }

        #endregion //Business Properties and Methods

        #region Validation Rules
        private void AddCustomRules()
        {
            //add custom/non-generated rules here...
        }

        private void AddCommonRules()
        {
            //
            // UserName
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "UserName");
            ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("UserName", 50));
            //
            // UserDisplayName
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "UserDisplayName");
            ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("UserDisplayName", 50));
            //
            // PasswordHash
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "PasswordHash");
            ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("PasswordHash", 50));
            //
            // Salt
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "Salt");
            ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("Salt", 50));
            //
            // Email
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "Email");
            ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("Email", 50));
            //
            // CreationDate
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "CreationDateString");
            //
            // ValidFrom
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "ValidFromString");
            //
            // ValidTo
            //
            ValidationRules.AddRule(CommonRules.StringRequired, "ValidToString");
            //
            // ActivationKey
            //
            ValidationRules.AddRule(CommonRules.StringMaxLength, new CommonRules.MaxLengthRuleArgs("ActivationKey", 100));
        }

        protected override void AddBusinessRules()
        {
            AddCommonRules();
            AddCustomRules();
        }
        #endregion //Validation Rules

        #region Authorization Rules
        protected override void AddAuthorizationRules()
        {
            //TODO: Define authorization rules in JmdUser
            //AuthorizationRules.AllowRead("Id", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("SiteId", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("UserName", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("UserDisplayName", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("PasswordHash", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("Salt", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("IsInternal", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("Email", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("Status", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("CreationDate", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("CreationDateString", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("ValidFrom", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("ValidFromString", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("ValidTo", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("ValidToString", "JmdUserReadGroup");
            //AuthorizationRules.AllowRead("ActivationKey", "JmdUserReadGroup");

            //AuthorizationRules.AllowWrite("SiteId", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("UserName", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("UserDisplayName", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("PasswordHash", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("Salt", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("IsInternal", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("Email", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("Status", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("CreationDateString", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("ValidFromString", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("ValidToString", "JmdUserWriteGroup");
            //AuthorizationRules.AllowWrite("ActivationKey", "JmdUserWriteGroup");
        }


        public static bool CanGetObject()
        {
            //TODO: Define CanGetObject permission in JmdUser
            return true;
            //if (Csla.ApplicationContext.User.IsInRole("JmdUserViewGroup"))
            //	return true;
            //return false;
        }

        public static bool CanAddObject()
        {
            //TODO: Define CanAddObject permission in JmdUser
            return true;
            //if (Csla.ApplicationContext.User.IsInRole("JmdUserAddGroup"))
            //	return true;
            //return false;
        }

        public static bool CanEditObject()
        {
            //TODO: Define CanEditObject permission in JmdUser
            return true;
            //if (Csla.ApplicationContext.User.IsInRole("JmdUserEditGroup"))
            //	return true;
            //return false;
        }

        public static bool CanDeleteObject()
        {
            //TODO: Define CanDeleteObject permission in JmdUser
            return true;
            //if (Csla.ApplicationContext.User.IsInRole("JmdUserDeleteGroup"))
            //	return true;
            //return false;
        }
        #endregion //Authorization Rules

        #region Factory Methods
        private _JmdUser()
        { /* require use of factory method */ }

        public static _JmdUser NewJmdUser()
        {
            if (!CanAddObject())
                throw new System.Security.SecurityException("User not authorized to add a JmdUser");
            return DataPortal.Create<_JmdUser>();
        }

        public static _JmdUser GetJmdUser(int id)
        {
            if (!CanGetObject())
                throw new System.Security.SecurityException("User not authorized to view a JmdUser");
            return DataPortal.Fetch<_JmdUser>(new Criteria(id));
        }

        public static void DeleteJmdUser(int id)
        {
            if (!CanDeleteObject())
                throw new System.Security.SecurityException("User not authorized to remove a JmdUser");
            DataPortal.Delete(new Criteria(id));
        }

        public override _JmdUser Save()
        {
            if (IsDeleted && !CanDeleteObject())
                throw new System.Security.SecurityException("User not authorized to remove a JmdUser");
            else if (IsNew && !CanAddObject())
                throw new System.Security.SecurityException("User not authorized to add a JmdUser");
            else if (!CanEditObject())
                throw new System.Security.SecurityException("User not authorized to update a JmdUser");

            return base.Save();
        }

        #endregion //Factory Methods

        #region Data Access

        #region Criteria

        [Serializable()]
        private class Criteria
        {
            public int Id;

            public Criteria(int id)
            {
                this.Id = id;
            }
        }

        #endregion //Criteria

        #region Data Access - Create
        [RunLocal]
        protected override void DataPortal_Create()
        {
            ValidationRules.CheckRules();
        }

        #endregion //Data Access - Create

        #region Data Access - Fetch

        [Transactional(TransactionalTypes.TransactionScope)]
        private void DataPortal_Fetch(Criteria criteria)
        {
            //Database db = DatabaseFactory.CreateDatabase();

            //string sqlCommand = "JMD_SITE_SelectAll";

            //using (DbCommand dbCommand = db.GetStoredProcCommand(sqlCommand))
            //{
                
            //    AddInsertParameters(db, dbCommand);
            //    db.ExecuteNonQuery(dbCommand);
            //}

            
            //using (SqlConnection cn = new SqlConnection(Database.JMDCMSConnection))
            //{
            //    cn.Open();

            //    ExecuteFetch(cn, criteria);
            //}//using
        }

        private void ExecuteFetch(SqlConnection cn, Criteria criteria)
        {
            using (SqlCommand cm = cn.CreateCommand())
            {
                cm.CommandType = CommandType.StoredProcedure;
                cm.CommandText = "GetJmdUser";

                cm.Parameters.AddWithValue("@ID", criteria.Id);

                using (SafeDataReader dr = new SafeDataReader(cm.ExecuteReader()))
                {
                    FetchObject(dr);
                    ValidationRules.CheckRules();

                    //load child object(s)
                    FetchChildren(dr);
                }
            }//using
        }

        private void FetchObject(SafeDataReader dr)
        {
            dr.Read();
            _id = dr.GetInt32("ID");
            _siteId = dr.GetInt32("SITE_ID");
            _userName = dr.GetString("USER_NAME");
            _userDisplayName = dr.GetString("USER_DISPLAY_NAME");
            _passwordHash = dr.GetString("PASSWORD_HASH");
            _salt = dr.GetString("SALT");
            _isInternal = dr.GetInt32("IS_INTERNAL");
            _email = dr.GetString("EMAIL");
            _status = dr.GetInt32("STATUS");
            _creationDate = dr.GetSmartDate("CREATION_DATE", _creationDate.EmptyIsMin);
            _validFrom = dr.GetSmartDate("VALID_FROM", _validFrom.EmptyIsMin);
            _validTo = dr.GetSmartDate("VALID_TO", _validTo.EmptyIsMin);
            _activationKey = dr.GetString("ACTIVATION_KEY");
        }

        private void FetchChildren(SafeDataReader dr)
        {
        }
        
        #endregion //Data Access - Fetch

        #region Data Access - Insert
        [Transactional(TransactionalTypes.TransactionScope)]
        protected override void DataPortal_Insert()
        {
            //using (SqlConnection cn = new SqlConnection(Database.JMDCMSConnection))
            //{
            //    cn.Open();

            //    ExecuteInsert(cn);

            //    //update child object(s)
            //    UpdateChildren(cn);
            //}//using

        }

        private void ExecuteInsert(SqlConnection cn)
        {
            using (SqlCommand cm = cn.CreateCommand())
            {
                cm.CommandType = CommandType.StoredProcedure;
                cm.CommandText = "AddJmdUser";

                AddInsertParameters(cm);

                cm.ExecuteNonQuery();

                _id = (int)cm.Parameters["@NewID"].Value;
            }//using
        }

        private void AddInsertParameters(SqlCommand cm)
        {
            cm.Parameters.AddWithValue("@SITE_ID", _siteId);
            cm.Parameters.AddWithValue("@USER_NAME", _userName);
            cm.Parameters.AddWithValue("@USER_DISPLAY_NAME", _userDisplayName);
            cm.Parameters.AddWithValue("@PASSWORD_HASH", _passwordHash);
            cm.Parameters.AddWithValue("@SALT", _salt);
            cm.Parameters.AddWithValue("@IS_INTERNAL", _isInternal);
            cm.Parameters.AddWithValue("@EMAIL", _email);
            cm.Parameters.AddWithValue("@STATUS", _status);
            cm.Parameters.AddWithValue("@CREATION_DATE", _creationDate.DBValue);
            cm.Parameters.AddWithValue("@VALID_FROM", _validFrom.DBValue);
            cm.Parameters.AddWithValue("@VALID_TO", _validTo.DBValue);
            if (_activationKey.Length > 0)
                cm.Parameters.AddWithValue("@ACTIVATION_KEY", _activationKey);
            else
                cm.Parameters.AddWithValue("@ACTIVATION_KEY", DBNull.Value);
            cm.Parameters.AddWithValue("@NewID", _id);
            cm.Parameters["@NewID"].Direction = ParameterDirection.Output;
        }

        private void AddInsertParameters(DbCommand dbCommand)
        {

        }



        #endregion //Data Access - Insert

        #region Data Access - Update
        [Transactional(TransactionalTypes.TransactionScope)]
        protected override void DataPortal_Update()
        {
            //using (SqlConnection cn = new SqlConnection(Database.JMDCMSConnection))
            //{
            //    cn.Open();

            //    if (base.IsDirty)
            //    {
            //        ExecuteUpdate(cn);
            //    }

            //    //update child object(s)
            //    UpdateChildren(cn);
            //}//using

        }

        private void ExecuteUpdate(SqlConnection cn)
        {
            using (SqlCommand cm = cn.CreateCommand())
            {
                cm.CommandType = CommandType.StoredProcedure;
                cm.CommandText = "UpdateJmdUser";

                AddUpdateParameters(cm);

                cm.ExecuteNonQuery();

            }//using
        }

        private void AddUpdateParameters(SqlCommand cm)
        {
            cm.Parameters.AddWithValue("@ID", _id);
            cm.Parameters.AddWithValue("@SITE_ID", _siteId);
            cm.Parameters.AddWithValue("@USER_NAME", _userName);
            cm.Parameters.AddWithValue("@USER_DISPLAY_NAME", _userDisplayName);
            cm.Parameters.AddWithValue("@PASSWORD_HASH", _passwordHash);
            cm.Parameters.AddWithValue("@SALT", _salt);
            cm.Parameters.AddWithValue("@IS_INTERNAL", _isInternal);
            cm.Parameters.AddWithValue("@EMAIL", _email);
            cm.Parameters.AddWithValue("@STATUS", _status);
            cm.Parameters.AddWithValue("@CREATION_DATE", _creationDate.DBValue);
            cm.Parameters.AddWithValue("@VALID_FROM", _validFrom.DBValue);
            cm.Parameters.AddWithValue("@VALID_TO", _validTo.DBValue);
            if (_activationKey.Length > 0)
                cm.Parameters.AddWithValue("@ACTIVATION_KEY", _activationKey);
            else
                cm.Parameters.AddWithValue("@ACTIVATION_KEY", DBNull.Value);
        }

        private void UpdateChildren(SqlConnection cn)
        {
        }
        #endregion //Data Access - Update

        #region Data Access - Delete
        [Transactional(TransactionalTypes.TransactionScope)]
        protected override void DataPortal_DeleteSelf()
        {
            DataPortal_Delete(new Criteria(_id));
        }

        [Transactional(TransactionalTypes.TransactionScope)]
        private void DataPortal_Delete(Criteria criteria)
        {
            //using (SqlConnection cn = new SqlConnection(Database.JMDCMSConnection))
            //{
            //    cn.Open();

            //    ExecuteDelete(cn, criteria);

            //}//using

        }

        private void ExecuteDelete(SqlConnection cn, Criteria criteria)
        {
            using (SqlCommand cm = cn.CreateCommand())
            {
                cm.CommandType = CommandType.StoredProcedure;
                cm.CommandText = "DeleteJmdUser";

                cm.Parameters.AddWithValue("@ID", criteria.Id);

                cm.ExecuteNonQuery();
            }//using
        }
        #endregion //Data Access - Delete
        #endregion //Data Access
    }
}


