DROP PROCEDURE [dbo].[JMD_PAGE_MOD_INIT]
GO
CREATE PROCEDURE [dbo].[JMD_PAGE_MOD_INIT]
  @Site_URL nvarchar(255)
AS

/*
	Copyright (C) 2012  Ajay Handa - jmdcms.com 
	This program is free software; 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	License for more details.
	Some rights reserved. Please read License.txt for details.
	Removing above copyright message is a violation of license terms.

	Name: JMD_PAGE_MOD_INIT
	Author: Ajay Handa
	Purpose: Initializes default modules on default page at site creation.
	
	Modifications:
	WI:783 - Admin;Guests Access Changed to Admin;Guests;Registered 
    WI:650 - Add admin module, Default values assigned to page header 
    WI:650: Module manager module removed, will be added in next version
    powered by message removed, Footer added default text footer.
    
*/

  Declare
    @Site_Id int,
    @Mod_Id int,
    @Div_Id int,
    @PageModId int,
    @Logo varchar(250)

  Select
    @Site_Id = dbo.JMD_GET_SITE_ID(@Site_URL)  

  /***************************************************************************
    Page: Login
    Module: Login.ascx
    Div: Center
  */
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'Login.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'login', @Mod_Id, 'Login',  @Div_Id, 1, 0, 
      'Admin;Guests;Registered', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: NewUser
    Module: NewUser.ascx
    Div: Center
  */
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'NewUser.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'NewUser', @Mod_Id, 'New User',  @Div_Id, 1, 0, 
      'Admin;Guests;Registered', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: AddPage
    Module: AddPage.ascx
    Div: MiddleFull
  */   
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'AddPage.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'AddPage', @Mod_Id, 'Add Page',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: AddModule
    Module: AddModule.ascx
    Div: MiddleFull
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'AddModule.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'AddModule', @Mod_Id, 'Add Module',  @Div_Id, 1, 0, 
      'Admin;Guests;Registered', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: JMDHTMLEdit
    Module: JMDHTMLEdit.ascx
    Div: MiddleFull
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTMLEdit.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'JMDHTMLEdit', @Mod_Id, 'Edit Module Contents',  @Div_Id, 1, 0, 
      'Admin;Registered', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: Default
    Module: MenuTop.ascx
    Div: Top
  */ 
  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Top'

  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'MenuTop.ascx'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Default', @Mod_Id, 'MenuTop',  @Div_Id, 1, 1, 
      'Admin;Guests;Registered;', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: Default
    Module: JMDHTML.ascx
    Div: Header
  */ 
  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Header'

  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTML.ascx'

  Execute
    @PageModId = JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Default', @Mod_Id, ' ',  @Div_Id, 1, 1, 
      'Admin;Guests;Registered;', 'Admin','Admin','Admin','Admin' , null, 1

  Set @Logo = '[br][br][h1]'+@Site_URL+'[/h1][br][br]'

  Execute
    JMD_MOD_HTML_SAVE
      @PageModId, --@Page_Mod_Id int,
      '', --@Title nvarchar(255),
      @Logo --@HTML_Text Text

  /***************************************************************************
    Page: Default
    Module: MenuMain.ascx
    Div: MainMenu
  */ 
  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'MainMenu'

  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'MenuMain.ascx'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Default', @Mod_Id, 'MenuMain',  @Div_Id, 1, 1, 
      'Admin;Guests;Registered;', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: Default
    Module: BreadCrumbs.ascx
    Div: SubHeader
  */ 
  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'SubHeader'

  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'BreadCrumbs.ascx'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Default', @Mod_Id, 'BreadCrumbs',  @Div_Id, 1, 1, 
      'Admin;Guests;Registered;', 'Admin','Admin','Admin','Admin' , null, 1



  /***************************************************************************
    Page: BrowseForms
    Module: BrowseForms.ascx
    Div: Center
  */ 
  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'BrowseForms.ascx'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'BrowseForms', @Mod_Id, 'BrowseForms',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: BrowseForms
    Module: BrowseForms.ascx
    Div: Center
  */ 
  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'NewForm.ascx'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'NewForm', @Mod_Id, 'NewForm',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin' , null, 1


  /***************************************************************************
    Page: NewCol
    Module: NewCol.ascx
    Div: Center
  */ 
  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'NewCol.ascx'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'NewCol', @Mod_Id, 'NewCol',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin' , null, 1

 /***************************************************************************
    Page: BrowseCols
    Module: BrowseCols.ascx
    Div: Center
  */ 
  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'BrowseCols.ascx'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'BrowseCols', @Mod_Id, 'BrowseCols',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin' , null, 1

/***************************************************************************
    Page: NewRecord
    Module: NewRecord.ascx
    Div: Center
  */ 
  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'NewRecord.ascx'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'NewRecord', @Mod_Id, 'NewRecord',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin', null, 1

/***************************************************************************
    Page: DF
    Module: DynamicForms.ascx
    Div: Center
  */ 
  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'DynamicForms.ascx'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'DF', @Mod_Id, 'DF',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin' , null, 1

/***************************************************************************
    Page: LookupDialog
    Module: LookupDialog.ascx
    Div: Center
  */ 
  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'LookupDialog.ascx'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'LookupDialog', @Mod_Id, 'LookupDialog',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin' , null, 1


  /***************************************************************************
    Page: DeletePage
    Module: DeletePage.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'DeletePage.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'DeletePage', @Mod_Id, 'Delete Page',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: Default
    Module: JMDHTML.ascx
    Div: Center
    Purpose: Welcome Message with initial site creation.
  */ 
  /*
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTML.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Default', @Mod_Id, 'Welcome',  @Div_Id, 1, 0, 
      'Admin;Guests;Registered', 'Admin','Admin','Admin','Admin', null, 1 
*/
  /***************************************************************************
    Page: Default
    Module: JMDHTML.ascx
    Div: Footer
    Purpose: Footer HTML Init
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTML.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Footer'

  Execute
    @PageModId = JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Default', @Mod_Id, ' ',  @Div_Id, 1, 1, 
      'Admin;Guests;Registered', 'Admin','Admin','Admin','Admin' , null, 1

  Execute
    JMD_MOD_HTML_SAVE
      @PageModId, --@Page_Mod_Id int,
      '', --@Title nvarchar(255),
      'FOOTER' --@HTML_Text Text
      

  /***************************************************************************
    Page: Default
    Module: JMDHTML.ascx
    Div: PoweredBy
    Purpose: Powered By Message
  */ 
/*
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTML.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'PoweredBy'


  Execute
    @PageModId = JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Default', @Mod_Id, ' ',  @Div_Id, 1, 1, 
      'Admin;Guests;Registered', 'Admin','Admin','Admin','Admin' 

  Execute
    JMD_MOD_HTML_SAVE
      @PageModId, --@Page_Mod_Id int,
      '', --@Title nvarchar(255),
      'powered by <a href="http://www.jmdcms.com">JMDCMS.com</a><br />copyright  2007 ' --@HTML_Text Text

*/

  /***************************************************************************
    Page: DeletePage
    Module: DeletePage.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'DeletePageModule.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'DeletePageModule', @Mod_Id, 'Delete Page Module',  @Div_Id, 1, 0, 
      'Admin;Registered', 'Admin','Admin','Admin','Admin' , null, 1


  /***************************************************************************
    Page: Users
    Module: Users.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'Users.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Users', @Mod_Id, 'Users Admin',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1
      
  /***************************************************************************
    Page: Roles
    Module: JMDHTMLMenuLeft.ascx
    Div: Left
  */ 

  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTMLMenuLeft.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Left'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Users', @Mod_Id, 'Admin',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1
      

  /***************************************************************************
    Page: DeleteUser
    Module: DeleteUser.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'DeleteUser.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'DeleteUser', @Mod_Id, 'Delete User',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1





  /***************************************************************************
    Page: Pages
    Module: Pages.ascx
    Div: Center
  */ 
  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'Pages.ascx'

  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'Pages', @Mod_Id, 'Pages Admin',  @Div_Id, 1, 0, 
  --    'Admin', 'Admin','Admin','Admin','Admin' , null, 1


  /***************************************************************************
    Page: Modules
    Module: Modules.ascx
    Div: Center
    WI:650: Module manager module removed, will be added in next version
  */ 
/*
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'Modules.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Modules', @Mod_Id, 'Modules Admin',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' 
*/
  /***************************************************************************
    Page: Roles
    Module: Roles.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'Roles.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Roles', @Mod_Id, 'Roles Admin',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1
      
  /***************************************************************************
    Page: Roles
    Module: JMDHTMLMenuLeft.ascx
    Div: Left
  */ 

  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTMLMenuLeft.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Left'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Roles', @Mod_Id, 'Admin',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1
      

  /***************************************************************************
    Page: AddRole
    Module: AddRole.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'AddRole.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'AddRole', @Mod_Id, 'Manage Roles',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: DeleteRole
    Module: DeleteRole.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'DeleteRole.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'DeleteRole', @Mod_Id, 'Delete Role',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1

  /***************************************************************************
    Page: UserRoles
    Module: UserRoles.ascx
    Div: Center
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'UserRoles.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'UserRoles', @Mod_Id, 'User Roles',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1


  /***************************************************************************
    Page: SMO
    Module: SMO.ascx
    Div: Center
  */ 
  --Select
  --  @Mod_Id = Id
  --From
  --  JMD_MOD
  --Where
  --  Site_Id = @Site_Id and
  --  Mod_Name = 'SMO.ascx'

  --Select
  --  @Div_Id = Id
  --From
  --  JMD_DIV
  --Where
  --  Site_Id = @Site_Id and
  --  Div_Name = 'Center'

  --Execute
  --  JMD_PAGE_MOD_SAVE
  --    '0',  @Site_URL, 'SMO', @Mod_Id, 'SQL Server Management',  @Div_Id, 1, 0, 
  --    'Admin;Registered', 'Admin','Admin','Admin','Admin' , null, 1


  /***************************************************************************
    Page: Admin
    Module: Admin.ascx
    Div: Center
    WI:650 - Add admin module
  */ 
  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'Admin.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Center'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Admin', @Mod_Id, 'Site Administration',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1

	Execute DBO.JMD_MOD_ADMIN_INIT
		@Site_URL

  /***************************************************************************
    Page: Admin
    Module: JMDHTMLMenuLeft.ascx
    Div: Left
  */ 

  Select
    @Mod_Id = Id
  From
    JMD_MOD
  Where
    Site_Id = @Site_Id and
    Mod_Name = 'JMDHTMLMenuLeft.ascx'

  Select
    @Div_Id = Id
  From
    JMD_DIV
  Where
    Site_Id = @Site_Id and
    Div_Name = 'Left'

  Execute
    JMD_PAGE_MOD_SAVE
      '0',  @Site_URL, 'Admin', @Mod_Id, 'Admin',  @Div_Id, 1, 0, 
      'Admin', 'Admin','Admin','Admin','Admin' , null, 1


GO