DROP PROCEDURE JMD_PAGE_SAVE
GO
/*****************************************************************************
  NAME: JMD_PAGE_SAVE
  PURPOSE: To Insert record into JMD_PAGE Table
*****************************************************************************/
CREATE PROCEDURE [dbo].[JMD_PAGE_SAVE]
  @Page_Id int,
  @Site_URL nvarchar(255),
  @PAGE_NAME nvarchar(255),
  @PAGE_CAPTION nvarchar(255),
  @META_TITLE nvarchar(255),
  @META_DESC nvarchar(1000),
  @META_KEYWORDS nvarchar(1000),
  @PARENT_PAGE_NAME nvarchar(255) = null,
  @IS_PUBLISHED int,
  @IS_ROOT int,
  @IS_MENU_ITEM int,
  @VIEW_ROLES NVARCHAR(1000),
  @ADD_ROLES NVARCHAR(1000),
  @EDIT_ROLES NVARCHAR(1000),
  @DELETE_ROLES NVARCHAR(1000),
  @MOVE_ROLES NVARCHAR(1000),
  @ADD_MODULE_ROLES NVARCHAR(1000),
  @IS_INTERNAL int,
  @Load_Shared_Modules int,
  @iPage_Sort Int = null,
  @Is_ViewState_Enabled Int = null

As
/*
	Copyright (C) 2007  Ajay Handa - jmdcms.com 
	This program is free software; 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	License for more details.
	Some rights reserved. Please read License.txt for details.
	Removing above copyright message is a violation of license terms.

	Name: JMD_PAGE_SAVE
	Author: Ajay Handa
	Purpose: Inserts or update the page record.
	Work Item ID: 651
	WI:979: Enable/Disable view state by Page
	
*/

  Declare
    @Site_Id int,
    @Parent_Page_Id int,
    @Return_Page_Id int,
    @Page_Sort int

  Select
    @Site_Id = dbo.JMD_GET_SITE_ID(@Site_URL)  


  if ( exists ( Select 1 From JMD_PAGE Where Site_ID = @Site_ID and Page_Name = @Page_Name ) ) And
	( @Page_Id <= 0 )
  begin
	Select
		@Return_Page_Id = @Page_Id
	From
		JMD_Page
	Where
      Site_Id = @Site_Id and
      Page_Name = @Page_Name
		
	return @Return_Page_ID
  end

  if ( isnull(@PARENT_PAGE_NAME,'') != '' )
  begin
    Select
      @Parent_Page_Id = Id
    From
      JMD_Page
    Where
      Site_Id = @Site_Id and
      Page_Name = @PARENT_PAGE_NAME
  end

  if not exists ( Select * From JMD_PAGE where Id = @Page_Id )
  begin

    Select
      @Page_Sort = Max(Page_Sort)
    From
      JMD_PAGE
    Where
      Site_Id = @Site_Id and
      Parent_Page_Id = @Parent_Page_Id

    if (@IS_ROOT = 1)
    begin
      Select @PARENT_PAGE_ID = null

      Select
         @Page_Sort = Max(Page_Sort)
       From
         JMD_PAGE
       Where
         Site_Id = @Site_Id and
         Is_Root = 1
    end

    Select
      @Page_Sort = Coalesce(@Page_Sort,0)+10
    Insert into JMD_PAGE
    (
      SITE_ID,
      PAGE_NAME,
      PAGE_CAPTION,
      META_TITLE,
      META_DESC,
      META_KEYWORDS,
      PARENT_PAGE_ID,
      IS_PUBLISHED,
      IS_ROOT,
      IS_MENU_ITEM,
      VIEW_ROLES,
      ADD_ROLES,
      EDIT_ROLES,
      DELETE_ROLES,
      MOVE_ROLES,
      ADD_MODULE_ROLES,
      IS_INTERNAL,
      PAGE_SORT,
      Load_Shared_Modules,
      Is_ViewState_Enabled
    )
    values
    (
      @SITE_ID,
      @PAGE_NAME,
      @PAGE_CAPTION,
      @META_TITLE,
      @META_DESC,
      @META_KEYWORDS,
      @PARENT_PAGE_ID,
      @IS_PUBLISHED,
      @IS_ROOT,
      @IS_MENU_ITEM,
      @VIEW_ROLES,
      @ADD_ROLES,
      @EDIT_ROLES,
      @DELETE_ROLES,
      @MOVE_ROLES,
      @ADD_MODULE_ROLES,
      @IS_INTERNAL,
      @PAGE_SORT,
      @Load_Shared_Modules,
      Coalesce(@Is_ViewState_Enabled,1)
    )
    Select
      @Return_Page_Id = @@Identity
  end
  else
  begin
    Update
      JMD_PAGE
    Set
      SITE_ID = @SITE_ID,
      PAGE_NAME = @PAGE_NAME,
      PAGE_CAPTION = @PAGE_CAPTION,
      META_TITLE = @META_TITLE,
      META_DESC = @META_DESC,
      META_KEYWORDS = @META_KEYWORDS ,
      PARENT_PAGE_ID = @PARENT_PAGE_ID,
      IS_PUBLISHED = @IS_PUBLISHED,
      IS_ROOT = @IS_ROOT,
      IS_MENU_ITEM = @IS_MENU_ITEM,
      VIEW_ROLES = @VIEW_ROLES,
      ADD_ROLES = @ADD_ROLES,
      EDIT_ROLES = @EDIT_ROLES, 
      DELETE_ROLES = @DELETE_ROLES,
      MOVE_ROLES = @MOVE_ROLES,
      ADD_MODULE_ROLES = @ADD_MODULE_ROLES,
      IS_INTERNAL = @IS_INTERNAL,
      Load_Shared_Modules = @Load_Shared_Modules,
      Page_Sort = @iPage_Sort,
      Is_ViewState_Enabled = Coalesce(@Is_ViewState_Enabled,1)

    Where
      Id = @Page_Id
    Select
      @Return_Page_Id = Coalesce(@Page_Id,0)
  end

 Select Return_Page_Id = @Return_Page_Id
 return

 GO