DROP PROCEDURE [dbo].[JMD_SITE_CREATE]
GO
/*****************************************************************************
  NAME: JMD_SITE_CREATE
  PURPOSE: Site Creation...
*****************************************************************************/
CREATE PROCEDURE [dbo].[JMD_SITE_CREATE]
  @SITE_URL nvarchar(255),
  @Email nvarchar(50),
  @STYLE_SHEET nvarchar(255) = 'Userfiles/Style/JMDCMS/jmdcms.css',
  @SITE_VERSION nvarchar(20) = '',
  @SITE_FOLDER nvarchar(255) = '',
  @User_Name nvarchar(50) = null,  
  @Password_Hash nvarchar(50) = null,
  @Salt nvarchar(50) = null

As
/*
	Copyright (C) 2007  Ajay Handa - jmdcms.com 
	This program is free software; 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	License for more details.
	Some rights reserved. Please read License.txt for details.
	Removing above copyright message is a violation of license terms.

	Author: Ajay Handa
	Purpose: New Site Creation
	WI:650:handaajay15:Mar/18/07: Transaction Implemented
*/
SET XACT_ABORT ON
BEGIN TRANSACTION

  Declare
    @Site_Id int,
    @Page_Id int

  /*
    Table: JMD_SITE
    Check and Insert new site record.
  */
  Execute 
    @Site_Id = JMD_SITE_INSERT @SITE_URL, null, 
      @STYLE_SHEET, @SITE_VERSION, @SITE_FOLDER

  /***************************************************************************
    Table: JMD_MOD
    Check and Insert Default Roles.
  */
  Execute 
    JMD_MOD_INIT @Site_URL
  /***************************************************************************
    Table: JMD_DIV
    Check and Insert DIVS.
  */
  Execute 
    JMD_DIV_INIT @Site_URL
  /***************************************************************************
    Table: JMD_ROLE
    Check and Insert Default Roles.
  */
  Execute 
    JMD_ROLE_INIT @Site_URL

  /***************************************************************************
    Table: JMD_USER
    Check and Insert Default Users.
  */
	
	if ( IsNull(@User_Name,'') <> '')
	begin	
		Execute 
		  JMD_USER_INIT @Site_URL, @Password_Hash, 
		  @Salt, @Email, @User_Name
	end
	else
	begin
		Execute 
		  JMD_USER_INIT @Site_URL, '57B71C1931CDCE26AC585E9FEC647B480C924D76', 
		  'AHNWuE0=', @Email
	end	  
  /*
    Table: JMD_USER_ROLE
    Check and Insert Default User Roles.
  */
  
	if ( IsNull(@User_Name,'') <> '')
	begin	
		Execute 
		  JMD_USER_ROLE_INIT @Site_URL, @User_Name
	end
	else
	begin
		Execute 
		  JMD_USER_ROLE_INIT @Site_URL
	end	  
      
  /*
    Table: JMD_PAGE
    Check and Insert Default page.
  */
    Execute 
      JMD_PAGE_INIT @Site_URL
  /*
    Table: JMD_PAGE_MOD
    Check and Insert DEFAULT MODULES ON PAGE
  */
    Execute 
      JMD_PAGE_MOD_INIT @Site_URL

    Execute 
      JMD_STYLE_INIT @Site_URL

    Execute 
      JMD_CONTENT_TEMPLATE_INIT @Site_URL


	Select
		*
	From
		JMD_SITE
	Where
		Id = @Site_Id

COMMIT
GO