--region Drop Existing Procedures

IF OBJECT_ID(N'[dbo].[JMD_MOD_WEBHOST_InsertUpdate]') IS NOT NULL
	DROP PROCEDURE [dbo].[JMD_MOD_WEBHOST_InsertUpdate]

IF OBJECT_ID(N'[dbo].[JMD_MOD_WEBHOST_Delete]') IS NOT NULL
	DROP PROCEDURE [dbo].[JMD_MOD_WEBHOST_Delete]

IF OBJECT_ID(N'[dbo].[JMD_MOD_WEBHOST_Select]') IS NOT NULL
	DROP PROCEDURE [dbo].[JMD_MOD_WEBHOST_Select]

IF OBJECT_ID(N'[dbo].[JMD_MOD_WEBHOST_SelectAll]') IS NOT NULL
	DROP PROCEDURE [dbo].[JMD_MOD_WEBHOST_SelectAll]

--endregion

GO

--region [dbo].[JMD_MOD_WEBHOST_InsertUpdate]


CREATE PROCEDURE [dbo].[JMD_MOD_WEBHOST_InsertUpdate]
	@ID uniqueidentifier,
	@PAGE_MOD_ID int,
	@NAME nvarchar(50),
	@URL nvarchar(100),
	@BUSINESS_START_DATE datetime,
	@CUSTOMER_COUNT int,
	@SUPPORT nvarchar(500),
	@DATA_CENTER nvarchar(500),
	@HOSTING_TYPE uniqueidentifier,
	@OS_TYPE uniqueidentifier,
	@USER_ID int,
	@LAST_CHANGED datetime,
	@TOTAL_CLICKS int,
	@AFFILIATION_URL nvarchar(1000)
AS

/*
Copyright (C) 2007  Ajay Handa - jmdcms.com
This program is free software;
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
License for more details.
Some rights reserved. Please read License.txt for details.
Removing above copyright message is a violation of license terms.
*/

SET NOCOUNT ON

IF EXISTS(SELECT [ID] FROM [dbo].[JMD_MOD_WEBHOST] WHERE [ID] = @ID)
BEGIN
	UPDATE [dbo].[JMD_MOD_WEBHOST] SET
		[PAGE_MOD_ID] = @PAGE_MOD_ID,
		[NAME] = @NAME,
		[URL] = @URL,
		[BUSINESS_START_DATE] = @BUSINESS_START_DATE,
		[CUSTOMER_COUNT] = @CUSTOMER_COUNT,
		[SUPPORT] = @SUPPORT,
		[DATA_CENTER] = @DATA_CENTER,
		[HOSTING_TYPE] = @HOSTING_TYPE,
		[OS_TYPE] = @OS_TYPE,
		[USER_ID] = @USER_ID,
		[LAST_CHANGED] = @LAST_CHANGED,
		[TOTAL_CLICKS] = @TOTAL_CLICKS,
		[AFFILIATION_URL] = @AFFILIATION_URL
	WHERE
		[ID] = @ID
END
ELSE
BEGIN
	INSERT INTO [dbo].[JMD_MOD_WEBHOST] (
		[ID],
		[PAGE_MOD_ID],
		[NAME],
		[URL],
		[BUSINESS_START_DATE],
		[CUSTOMER_COUNT],
		[SUPPORT],
		[DATA_CENTER],
		[HOSTING_TYPE],
		[OS_TYPE],
		[USER_ID],
		[LAST_CHANGED],
		[TOTAL_CLICKS],
		[AFFILIATION_URL]
	) VALUES (
		@ID,
		@PAGE_MOD_ID,
		@NAME,
		@URL,
		@BUSINESS_START_DATE,
		@CUSTOMER_COUNT,
		@SUPPORT,
		@DATA_CENTER,
		@HOSTING_TYPE,
		@OS_TYPE,
		@USER_ID,
		@LAST_CHANGED,
		@TOTAL_CLICKS,
		@AFFILIATION_URL
	)
END

SELECT * FROM [dbo].[JMD_MOD_WEBHOST]

--endregion

GO

--region [dbo].[JMD_MOD_WEBHOST_Delete]


CREATE PROCEDURE [dbo].[JMD_MOD_WEBHOST_Delete]
	@ID uniqueidentifier
AS

/*
Copyright (C) 2007  Ajay Handa - jmdcms.com
This program is free software;
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
License for more details.
Some rights reserved. Please read License.txt for details.
Removing above copyright message is a violation of license terms.
*/

SET NOCOUNT ON

DELETE FROM [dbo].[JMD_MOD_WEBHOST]
WHERE
	[ID] = @ID

--endregion

GO

--region [dbo].[JMD_MOD_WEBHOST_Select]


CREATE PROCEDURE [dbo].[JMD_MOD_WEBHOST_Select]
	@ID uniqueidentifier
AS

/*
Copyright (C) 2007  Ajay Handa - jmdcms.com
This program is free software;
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
License for more details.
Some rights reserved. Please read License.txt for details.
Removing above copyright message is a violation of license terms.
*/

SET NOCOUNT ON
SET TRANSACTION ISOLATION LEVEL READ COMMITTED

SELECT
	[ID],
	[PAGE_MOD_ID],
	[NAME],
	[URL],
	[BUSINESS_START_DATE],
	[CUSTOMER_COUNT],
	[SUPPORT],
	[DATA_CENTER],
	[HOSTING_TYPE],
	[OS_TYPE],
	[USER_ID],
	[LAST_CHANGED],
	[TOTAL_CLICKS],
	[AFFILIATION_URL]
FROM
	[dbo].[JMD_MOD_WEBHOST]
WHERE
	[ID] = @ID

--endregion

GO

--region [dbo].[JMD_MOD_WEBHOST_SelectAll]


CREATE PROCEDURE [dbo].[JMD_MOD_WEBHOST_SelectAll]
AS

/*
Copyright (C) 2007  Ajay Handa - jmdcms.com
This program is free software;
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
License for more details.
Some rights reserved. Please read License.txt for details.
Removing above copyright message is a violation of license terms.
*/

SET NOCOUNT ON
SET TRANSACTION ISOLATION LEVEL READ COMMITTED

SELECT
	[ID],
	[PAGE_MOD_ID],
	[NAME],
	[URL],
	[BUSINESS_START_DATE],
	[CUSTOMER_COUNT],
	[SUPPORT],
	[DATA_CENTER],
	[HOSTING_TYPE],
	[OS_TYPE],
	[USER_ID],
	[LAST_CHANGED],
	[TOTAL_CLICKS],
	[AFFILIATION_URL]
FROM
	[dbo].[JMD_MOD_WEBHOST]

--endregion

GO

