/*
	Copyright (C) 2007  Ajay Handa - jmdcms.com 
	This program is free software; 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	License for more details.
	Some rights reserved. Please read License.txt for details.
	Removing above copyright message is a violation of license terms.
*/
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Text;
using System.Security.Principal;
using JMDCMS.Web.Lib;


public partial class Modules_Core_MenuMain : System.Web.UI.UserControl
{
    protected void Page_Load(object sender, EventArgs e)
    {
        LtMainMenu.Text = MainMenu();
    }

    private bool InsertMenu(StringBuilder sb, DataRow dr)
    {
        IPrincipal p = HttpContext.Current.User;
        string[] PageRoles = dr["View_Roles"].ToString().Split(new char[] { ';' });
        bool result = false;

        foreach (string Role in PageRoles)
        {
            string ActiveTag = "current";
            string MenuLink = "<li class=\"{2}\"><a class=\"{2}\"  href=\"{0}.aspx\"><span>{1}</span></a>";

            if (
                (p.IsInRole(Role) && (string.Compare(Role.Trim(), string.Empty) != 0)) ||
                (!(p.Identity.IsAuthenticated) && (String.Compare(Role, "Guests", true) == 0)) ||
                ((p.Identity.IsAuthenticated) && (String.Compare(Role, "Registered", true) == 0))
                )
            {
                if (string.Compare(Util.PageName(Request.QueryString["Pg"].ToString()),
                    dr["PAGE_NAME"].ToString(), true) == 0)
                {
                    ActiveTag = "current";
                }
                else
                {
                    ActiveTag = "";
                }

                if (
                    (dr["IS_PUBLISHED"].ToString() == "1") &&
                    (dr["IS_MENU_ITEM"].ToString() == "1") &&
                    (dr["IS_DELETED"].ToString() == "0")
                    )
                {
                    sb.AppendFormat(MenuLink,
                        dr["PAGE_NAME"].ToString(),
                        dr["PAGE_CAPTION"].ToString(),
                        ActiveTag).AppendLine();
                }
                result = true;
                break;
            }
        }
        return result;
    }

    private void BuildMenu(StringBuilder sb, DataRow[] rows, DataTable dt)
    {
        foreach (DataRow dr in rows)
        {
            if (InsertMenu(sb, dr))
            {
                var filterExpression = string.Format("PARENT_PAGE_ID={0} and IS_PUBLISHED=1 and IS_MENU_ITEM=1 and IS_DELETED=0 ", dr["ID"].ToString());
                var sortExpression = "PAGE_SORT";
                var childRows = dt.Select(filterExpression,sortExpression);
                if (childRows.Length > 0)
                {
                    sb.AppendFormat("<ul>").AppendLine();
                    BuildMenu(sb, childRows, dt);
                    sb.AppendFormat("</ul>").AppendLine();
                }
                sb.AppendFormat("</li>").AppendLine();
            }

        }
    }

    public string MainMenu()
    {
        StringBuilder sb = new StringBuilder();
        string ActiveTag = "current";
        string MenuLink = "<li class=\"{2}\"><a class=\"{2}\"  href=\"{0}.aspx\"><span>{1}</span></a></li>";
         
        DataTable dt;
        IPrincipal p = HttpContext.Current.User;
        string Legends = string.Empty;
        
        dt = SqlServer.JMD_MAIN_MENU(Util.SiteURL(Request.QueryString["Pg"].ToString()));

        if (dt.Rows.Count > 0)
        {
            sb.AppendFormat("<ul class=\"menu\">").AppendLine();
            foreach (DataRow dr in dt.Rows)
            {
                if (string.IsNullOrEmpty(dr["PARENT_PAGE_ID"].ToString()))
                {
                    BuildMenu(sb, new DataRow[] { dr }, dt);
                }
            }
            sb.AppendFormat("</ul>").AppendLine();
        }
        else
        {
            sb.AppendFormat("<td> No Menu Found...").AppendLine();
            sb.AppendFormat("</td>").AppendLine();
        }

        dt.Dispose();
        return sb.ToString();
    }
}
